import numpy


class Dimension(object):
    def __init__(self, length):
        self.length = length


class Variable(object):
    def __init__(self, data, dimensions=None, record=False, **kwargs):
        # fix strings, since netcdf3 has only the concept of char arrays
        if isinstance(data, basestring):
            data = list(data)

        # replace masked data --if any-- with missing_value.
        missing_value = (
                kwargs.get('missing_value') or
                kwargs.get('_FillValue') or 
                getattr(data, 'fill_value', None))
        if missing_value is not None:
            kwargs.setdefault('missing_value', missing_value)
            kwargs.setdefault('_FillValue', missing_value)
            self.data = numpy.ma.asarray(data).filled(missing_value)
        else:
            self.data = numpy.asarray(data)

        self.dimensions = dimensions
        self.record = record
        self.attributes = kwargs


class Group(object):
    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)


class NetCDF(object):
    @classmethod
    def save(klass, filename, **kwargs):
        # find netcdf loader
        if hasattr(klass, 'loader') and callable(getattr(klass, 'loader')):
            loader = getattr(klass, 'loader')
        else:
            from pupynere import netcdf_file as loader
        out = loader(filename, 'w', **kwargs)
        process(klass, out)
        out.close()


def process(obj, target):
    # add attributes
    for name in dir(obj):
        attr = getattr(obj, name)
        if isinstance(attr, basestring) and not name.startswith('_'):
            setattr(target, name, attr)

    # set variable names from the class
    for name in dir(obj):
        attr = getattr(obj, name)
        if isinstance(attr, (Variable, Dimension)):
            attr.name = name

    # add explicitly defined dimensions
    for name in dir(obj):
        attr = getattr(obj, name)
        if isinstance(attr, Dimension):
            target.createDimension(name, attr.length)

    # add groups
    for name in dir(obj):
        attr = getattr(obj, name)
        if isinstance(attr, Group):
            group = target.createGroup(name)
            process(attr, group)

    # add variables, and add their dimensions if necessary
    for name in dir(obj):
        attr = getattr(obj, name)
        if isinstance(attr, Variable):
            # add dimension?
            if attr.dimensions is None:
                attr.dimensions = [ attr ]
            for dim in attr.dimensions:
                if dim.name not in target.dimensions:
                    if dim.record:
                        target.createDimension(dim.name, None)
                    else:
                        target.createDimension(dim.name, len(dim.data))

            # create var
            if attr.data.dtype == numpy.int64:
                attr.data = attr.data.astype(numpy.int32)
            var = target.createVariable(
                    attr.name, 
                    attr.data.dtype, 
                    tuple(dim.name for dim in attr.dimensions))
            var[:] = attr.data[:]
            for k, v in attr.attributes.items():
                setattr(var, k, v)



RELEASE = True

from setuptools import setup, find_packages
import sys, os

classifiers = """\
Development Status :: 5 - Production/Stable
Environment :: Console
Intended Audience :: Developers
Intended Audience :: Science/Research
License :: OSI Approved :: MIT License
Operating System :: OS Independent
Programming Language :: Python
Topic :: Scientific/Engineering
Topic :: Software Development :: Libraries :: Python Modules
"""

version = '0.1.7'

setup(
        name='Puppy',
        version=version,
        description="DSL for creating NetCDF files",
        long_description="""\
A DSL for creating NetCDF files. Here's a simple example::

    from pup import *

    class Test(NetCDF):
        # NC_GLOBAL attributes go here
        history = 'Created for a test'

        # dimensions need to be set explicitly only when they
        # have no variable associated with them
        dim0 = Dimension(2)

        # variables that don't specify dimensions are assumed to
        # be their own dimension
        time = Variable(range(10), record=True, units='days since 2008-01-01')

        # now a variable with dimensions (time,)
        temperature = Variable(range(10), (time,), units='deg C')

    Test.save('simple.nc')

This will produce the following NetCDF file::

    netcdf simple {
    dimensions:
        dim0 = 2 ;
        time = UNLIMITED ; // (10 currently)
    variables:
        int time(time) ;
            time:units = "days since 2008-01-01" ;
        int temperature(time) ;
            temperature:units = "deg C" ;

    // global attributes:
            :history = "Created for a test" ;
    data:

     time = 0, 1, 2, 3, 4, 5, 6, 7, 8, 9 ;

     temperature = 0, 1, 2, 3, 4, 5, 6, 7, 8, 9 ;
    }

By default it uses pupynere for creating files, but this can be overloaded; we
can use the netCDF4 module, for example, which allows us to specify groups::

    from netCDF4 import Dataset

    class Test(NetCDF):
        loader = Dataset
        ...

        foo = Group(
            dim = Dimension(10),
            var = Variable(range(10)),
            ...
        )
    Test.save('simple.nc', format='NETCDF4')

Changelog:

:0.1.7:   Convert strings to array of chars.
:0.1.6:   Fix bug in dimension name.
:0.1.5:   Added support for Groups when using netcdf4.
:0.1.4:   Added support for masked arrays.
:0.1.3:   Pass keyword arguments in save() to the loader.
:0.1.2:   Improved optional loader detection.
:0.1.1:   Added pupynere dependency.
:0.1:     Initial release.

""",
        classifiers=filter(None, classifiers.split("\n")),
        keywords='netcdf data array math',
        author='Roberto De Almeida',
        author_email='roberto@dealmeida.net',
        url='http://bitbucket.org/robertodealmeida/puppy/',
        #download_url = "http://cheeseshop.python.org/packages/source/p/Puppy/Puppy-%s.tar.gz" % version,
        license='MIT',
        py_modules=['pup'],
        include_package_data=True,
        zip_safe=True,
        test_suite = 'nose.collector',
        install_requires=[
            'numpy',
            'pupynere',
        ],
        extras_require={
            'test': ['nose'],
        },
)
